#include <stdio.h>
#include <endian.h>
#include <stdint.h>

/**********************************************************************
 *
 * The following are support routines for inflate.c
 *
 **********************************************************************/

static uint32_t crc_32_tab[256];

/*
 * Code to compute the CRC-32 table. Borrowed from 
 * gzip-1.0.3/makecrc.c.
 */

static void
makecrc(void)
{
/* Not copyrighted 1990 Mark Adler	*/

  unsigned long c;      /* crc shift register */
  unsigned long e;      /* polynomial exclusive-or pattern */
  int i;                /* counter for all possible eight bit values */
  int k;                /* byte being shifted into crc apparatus */

  /* terms of polynomial defining this crc (except x^32): */
  static const int p[] = {0,1,2,4,5,7,8,10,11,12,16,22,23,26};

  /* Make exclusive-or pattern from polynomial */
  e = 0;
  for (i = 0; i < sizeof(p)/sizeof(int); i++)
    e |= 1L << (31 - p[i]);

  crc_32_tab[0] = 0;

  for (i = 1; i < 256; i++)
  {
    c = 0;
    for (k = i | 256; k != 1; k >>= 1)
    {
      c = c & 1 ? (c >> 1) ^ e : c >> 1;
      if (k & 1)
        c ^= e;
    }
    crc_32_tab[i] = c;
  }
}

unsigned long comp_crc(unsigned char *p, unsigned long len)
{
	unsigned long crc = 0xFFFFFFFFUL;

	while (len--) {
		crc = crc_32_tab[(crc ^ *p++) & 0xff] ^ (crc >> 8);
	}
	return crc ^ 0xFFFFFFFFUL;
}

int main(int argc, char **argv) {
	char *file, *output;
	int blocksize = 1024;
	unsigned char p[4 * 1024 * 1024];
	uint32_t len;
	makecrc();
	if (argc != 3) {
		fprintf(stderr, "Usage: %s input output\n", argv[0]);
		return 1;
	}
	file = argv[1];
	output = argv[2];
	FILE *fp = fopen(file, "r");
	FILE *out = fopen(output, "w");
	if (!fp) {
		fprintf(stderr, "Failing file %s\n", file);
		return 1;
	}
	if (!out) {
		fprintf(stderr, "Failed to open output file %s\n", output);
		return 1;
	}
	len = fread(p, 1, sizeof p, fp);
	uint32_t crc = comp_crc(p, len);
	fclose(fp);
	fwrite(p, 1, len, out);
	unsigned int padding = (2 * blocksize - len % blocksize - 12) % blocksize;
	/* seems mandatory if the last row contains data too */
	if (padding <= 4) padding += blocksize;
	while (padding--) {
		fputc(-1, out);
	}
	fwrite(&len, sizeof len, 1, out);
	fputc(0x78, out);
	fputc(0x56, out);
	fputc(0x34, out);
	fputc(0x12, out);
	fwrite(&crc, sizeof crc, 1, out);
	fclose(out);
	return 0;
}
